==> terms_export_import-1.0.3.tar.gz: terms_export_import/src/Service/TermsExportImportService.php <==
<?php

namespace Drupal\terms_export_import\Service;

use Drupal\taxonomy\Entity\Term;
use Drupal\taxonomy\Entity\Vocabulary;

/**
 * Service class for exporting and importing terms.
 */
class TermsExportImportService {

  /**
   * Export all terms for a given vocabulary as an array of simple data.
   *
   * @param string $vocabulary_id
   *   The machine name of the vocabulary to export.
   *
   * @return array
   *   An array of term data. Example:
   *   [
   *     [
   *       'name' => 'Term 1',
   *       'description' => 'Description for Term 1',
   *       'parent' => 'Parent Term Name',
   *       'weight' => 0,
   *     ],
   *     ...
   *   ]
   */
  public function exportTerms($vocabulary_id): array {
    $terms_data = [];
    $terms = \Drupal::entityTypeManager()->getStorage('taxonomy_term')
      ->loadTree($vocabulary_id);

    foreach ($terms as $term) {
      $term_obj = Term::load($term->tid);
      $parent_term = '';
      if (!empty($term->parents)) {
        // For simplicity, assume single parent (Drupal supports multiple).
        $parent_id = reset($term->parents);
        if ($parent_id != 0) {
          $parent_obj = Term::load($parent_id);
          $parent_term = $parent_obj ? $parent_obj->getName() : '';
        }
      }
      $terms_data[] = [
        'name' => $term_obj->getName(),
        'description' => $term_obj->getDescription(),
        'parent' => $parent_term,
        'weight' => $term_obj->getWeight(),
      ];
    }

    return $terms_data;
  }

  /**
   * Import terms from an array of data into a given vocabulary.
   *
   * @param string $vocabulary_id
   *   The machine name of the vocabulary to import into.
   * @param array $terms_data
   *   Array of terms data (same structure as export).
   *
   * @return void
   *   No return value.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function importTerms(string $vocabulary_id, array $terms_data): void {
    // Validate the vocabulary exists.
    $vocabulary = Vocabulary::load($vocabulary_id);
    if (!$vocabulary) {
      \Drupal::messenger()->addError('Invalid vocabulary ID.');
      return;
    }

    // Key terms by name for parent lookups.
    // Load existing terms to avoid duplicates or handle updates.
    $existing_terms = \Drupal::entityTypeManager()->getStorage('taxonomy_term')
      ->loadTree($vocabulary_id);
    $existing_map = [];
    foreach ($existing_terms as $term) {
      $existing_map[$term->name] = $term->tid;
    }

    foreach ($terms_data as $data) {
      $name = $data['name'] ?? '';
      $description = $data['description'] ?? '';
      $parent_name = $data['parent'] ?? '';
      $weight = $data['weight'] ?? 0;

      // Check if this term already exists.
      if (isset($existing_map[$name])) {
        // Update existing term if desired.
        $term_obj = Term::load($existing_map[$name]);
      }
      else {
        // Create a new term entity.
        $term_obj = Term::create([
          'name' => $name,
          'vid' => $vocabulary_id,
        ]);
      }

      // Set (or update) description and weight.
      $term_obj->setDescription($description);
      $term_obj->setWeight($weight);

      // Resolve parent if not empty.
      if ($parent_name && isset($existing_map[$parent_name])) {
        $term_obj->set('parent', [$existing_map[$parent_name]]);
      }
      else {
        $term_obj->set('parent', []);
      }

      $term_obj->save();

      // If new term, add it to $existing_map so child terms can reference it.
      if (!isset($existing_map[$name])) {
        $existing_map[$name] = $term_obj->id();
      }
    }
  }

  /**
   * Convert array of term data to CSV format and return as a string.
   */
  public function convertToCsv(array $terms_data): string {
    // First row: headers.
    $csv_rows = [];
    $headers = ['name', 'description', 'parent', 'weight'];
    $csv_rows[] = implode(',', $headers);

    foreach ($terms_data as $term) {
      $row = [
        $this->escapeCsv($term['name'] ?? ''),
        $this->escapeCsv($term['description'] ?? ''),
        $this->escapeCsv($term['parent'] ?? ''),
        $term['weight'] ?? 0,
      ];
      $csv_rows[] = implode(',', $row);
    }

    return implode("\n", $csv_rows);
  }

  /**
   * Parse CSV string into array of term data.
   */
  public function parseCsv($csv_string): array {
    $lines = explode("\n", trim($csv_string));
    $data = [];
    // Assume first line are headers.
    $headers = [];
    foreach ($lines as $index => $line) {
      $cells = str_getcsv($line);
      if ($index === 0) {
        $headers = $cells;
        continue;
      }
      $row = [];
      foreach ($headers as $i => $header) {
        $row[$header] = $cells[$i] ?? '';
      }
      $data[] = $row;
    }
    return $data;
  }

  /**
   * Simple CSV escaping for strings.
   */
  protected function escapeCsv($string) {
    // If string contains comma, quote, or new line, enclose in quotes.
    if (preg_match('/[,"\r\n]/', $string)) {
      $string = '"' . str_replace('"', '""', $string) . '"';
    }
    return $string;
  }

}

==> terms_export_import-1.0.6.tar.gz: terms_export_import/src/Service/TermsExportImportService.php <==
<?php

namespace Drupal\terms_export_import\Service;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\taxonomy\Entity\Term;

/**
 * Service class for exporting and importing terms.
 */
class TermsExportImportService {

  /**
   * Constructs a TermsExportImportService object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   */
  public function __construct(
    protected EntityTypeManagerInterface $entityTypeManager,
    protected MessengerInterface $messenger,
  ) {}

  /**
   * Export all terms for a given vocabulary as an array of simple data.
   *
   * @param string $vocabulary_id
   *   The machine name of the vocabulary to export.
   *
   * @return array
   *   An array of term data.
   */
  public function exportTerms(string $vocabulary_id): array {
    $terms_data = [];
    $term_storage = $this->entityTypeManager->getStorage('taxonomy_term');
    $terms = $term_storage->loadTree($vocabulary_id);

    foreach ($terms as $term) {
      $term_obj = $term_storage->load($term->tid);
      $parent_term = '';
      if (!empty($term->parents)) {
        $parent_id = reset($term->parents);
        if ($parent_id != 0) {
          $parent_obj = $term_storage->load($parent_id);
          $parent_term = $parent_obj ? $parent_obj->getName() : '';
        }
      }
      $terms_data[] = [
        'name' => $term_obj->getName(),
        'description' => $term_obj->getDescription(),
        'parent' => $parent_term,
        'weight' => $term_obj->getWeight(),
      ];
    }

    return $terms_data;
  }

  /**
   * Import terms from an array of data into a given vocabulary.
   *
   * @param string $vocabulary_id
   *   The machine name of the vocabulary to import into.
   * @param array $terms_data
   *   Array of terms data (same structure as export).
   *
   * @return void
   *   No return value.
   */
  public function importTerms(string $vocabulary_id, array $terms_data): void {
    $vocabulary_storage = $this->entityTypeManager->getStorage('taxonomy_vocabulary');
    $vocabulary = $vocabulary_storage->load($vocabulary_id);
    if (!$vocabulary) {
      $this->messenger->addError('Invalid vocabulary ID.');
      return;
    }

    $term_storage = $this->entityTypeManager->getStorage('taxonomy_term');
    $existing_terms = $term_storage->loadTree($vocabulary_id);
    $existing_map = [];
    foreach ($existing_terms as $term) {
      $existing_map[$term->name] = $term->tid;
    }

    foreach ($terms_data as $data) {
      $name = $data['name'] ?? '';
      $description = $data['description'] ?? '';
      $parent_name = $data['parent'] ?? '';
      $weight = $data['weight'] ?? 0;

      if (isset($existing_map[$name])) {
        $term_obj = $term_storage->load($existing_map[$name]);
      }
      else {
        $term_obj = Term::create([
          'name' => $name,
          'vid' => $vocabulary_id,
        ]);
      }

      $term_obj->setDescription($description);
      $term_obj->setWeight($weight);

      if ($parent_name && isset($existing_map[$parent_name])) {
        $term_obj->set('parent', [$existing_map[$parent_name]]);
      }
      else {
        $term_obj->set('parent', []);
      }

      $term_obj->save();

      if (!isset($existing_map[$name])) {
        $existing_map[$name] = $term_obj->id();
      }
    }
  }

  /**
   * Convert array of term data to CSV format and return as a string.
   *
   * @param array $terms_data
   *   Array of term data (same structure as export).
   *
   * @return string
   *   CSV string.
   */
  public function convertToCsv(array $terms_data): string {
    $csv_rows = [];
    $headers = ['name', 'description', 'parent', 'weight'];
    $csv_rows[] = implode(',', $headers);

    foreach ($terms_data as $term) {
      $row = [
        $this->escapeCsv($term['name'] ?? ''),
        $this->escapeCsv($term['description'] ?? ''),
        $this->escapeCsv($term['parent'] ?? ''),
        $term['weight'] ?? 0,
      ];
      $csv_rows[] = implode(',', $row);
    }

    return implode("\n", $csv_rows);
  }

  /**
   * Parse CSV string into array of term data.
   *
   * @param string $csv_string
   *   CSV string to parse.
   *
   * @return array
   *   Array of term data (same structure as export).
   */
  public function parseCsv(string $csv_string): array {
    $lines = explode("\n", trim($csv_string));
    $data = [];
    $headers = [];
    foreach ($lines as $index => $line) {
      $cells = str_getcsv($line);
      if ($index === 0) {
        $headers = $cells;
        continue;
      }
      $row = [];
      foreach ($headers as $i => $header) {
        $row[$header] = $cells[$i] ?? '';
      }
      $data[] = $row;
    }

    return $data;
  }

  /**
   * Simple CSV escaping for strings.
   *
   * @param string $string
   *   String to escape.
   *
   * @return string
   *   Escaped string.
   */
  protected function escapeCsv(string $string): string {
    if (preg_match('/[,"\r\n]/', $string)) {
      $string = '"' . str_replace('"', '""', $string) . '"';
    }

    return $string;
  }

}
